;*===========================================================================*
;= Mod2PSG V1.8                                                              =
;= PSG MOD include file for SMS/GG                                           =
;= by Martin Konrad (mkon@gmx.de)                                            =
;*===========================================================================*

; PSGMOD variables:
;PSGMOD_PATTERN_ADDRESS  = RAM_START     ;(word)
;PSGMOD_SONG_START       = RAM_START + 2 ;(word)
;PSGMOD_SONG_POSITION    = RAM_START + 4 ;(byte)
;PSGMOD_PATTERN_POSITION = RAM_START + 5 ;(byte)
;PSGMOD_CURRENT_SPEED    = RAM_START + 6 ;(byte)
;PSGMOD_SPEED_COUNTER    = RAM_START + 7 ;(byte)
;PSGMOD_SOUND_DISABLED   = RAM_START + 8 ;(byte)

; Use the following routines:
;
; ===== PSGMOD_INITIATE_NEW_SONG =====
; Input: HL = address of song
; ====================================
; Initiates a new song to be played.
; Call this one before you use PSGMOD_PLAY_NEXT_NOTE.
;
; ===== PSGMOD_PLAY_NEXT_NOTE =====
; No input required
; =================================
; Call this to play the song you have initiated using
; PSGMOD_INITIATE_NEW_SONG. It should be called every
; VBlank to get the correct speed.
;
; ===== PSGMOD_DISABLE_SOUND =====
; No input required
; ================================
; This will disable the sound. After this
; PSGMOD_PLAY_NEXT_NOTE won't play any sound.
;
; ===== PSGMOD_ENABLE_SOUND =====
; No input required
; ===============================
; This will enable the sound.
;
; ===== PSGMOD_TOGGLE_SOUND =====
; No input required
; ===============================
; This will toggle the sound on or off.
     
PSGMOD_INITIATE_NEW_SONG:
  xor a
  ld (PSGMOD_SONG_POSITION), a
  ld a, 64
  ld (PSGMOD_PATTERN_POSITION), a
  ld a, 1
  ld (PSGMOD_CURRENT_SPEED), a
  ld (PSGMOD_SPEED_COUNTER), a
  ld a, l
  ld (PSGMOD_SONG_START), a
  ld a, h
  ld (PSGMOD_SONG_START + 1), a
  ld ix, (PSGMOD_SONG_START)
  ld a, (IX)
  ld l, a
  ld h, 0
  call HL_512
  ld bc, 129
  add hl, bc
  ld bc, (PSGMOD_SONG_START)
  add hl, bc
  ld a, l
  ld (PSGMOD_PATTERN_ADDRESS), a
  ld a, h
  ld (PSGMOD_PATTERN_ADDRESS + 1), a
  ret

PSGMOD_PLAY_NEXT_NOTE:
  ld a, (PSGMOD_SOUND_DISABLED)
  cp 1
  ret Z

  ld hl, PSGMOD_SPEED_COUNTER
  dec (HL)
  ret NZ

  ld hl, (PSGMOD_PATTERN_ADDRESS)
  ld c, 3
  ld d, 0
PSGMOD_NEXT_CHANNEL:
  ld b, (HL)

  ld a, b
  and 15
  add a, 144
  add a, d
  out ($7F), a

  ld a, b
  srl a
  srl a
  srl a
  srl a
  add a, 128
  add a, d
  out ($7F), a
  inc hl
  ld a, (HL)
  out ($7F), a

  ld a, d
  add a, 32
  ld d, a
  inc hl
  dec c
  jr NZ, PSGMOD_NEXT_CHANNEL

  ld b, (HL)
  ld a, b
  and 15
  add a, 240
  out ($7F), a

  ld a, b
  srl a
  srl a
  srl a
  srl a
  or 224
  out ($7F), a

  ld hl, (PSGMOD_PATTERN_ADDRESS)
  ld bc, 7
  add hl, bc
  ld c, (HL)

  ld a, c
  and 128
  jr Z, PSGMOD_SET_SPEED
  ld a, 1
  ld (PSGMOD_PATTERN_POSITION), a
PSGMOD_SET_SPEED:
  ld a, c
  and 31
  ld (PSGMOD_CURRENT_SPEED), a
  ld (PSGMOD_SPEED_COUNTER), a

  ld hl, (PSGMOD_PATTERN_ADDRESS)
  ld bc, 8
  add hl, bc
  ld (PSGMOD_PATTERN_ADDRESS), hl

  ld hl, PSGMOD_PATTERN_POSITION
  dec (HL)
  ret NZ
  ld a, 64
  ld (PSGMOD_PATTERN_POSITION), a
  ld hl, PSGMOD_SONG_POSITION
  inc (HL)

  call PSGMOD_GET_PATTERN_NUMBER
  bit 7, d
  jr Z, PSGMOD_END_NOT_REACHED

  ld a, d
  and 127
  ld (PSGMOD_SONG_POSITION), a
  call PSGMOD_GET_PATTERN_NUMBER

PSGMOD_END_NOT_REACHED:
  ld h, 0
  ld l, d
  call HL_512
  ld bc, 129
  add hl, bc
  ld bc, (PSGMOD_SONG_START)
  add hl, bc
  ld a, l
  ld (PSGMOD_PATTERN_ADDRESS), a
  ld a, h
  ld (PSGMOD_PATTERN_ADDRESS + 1), a
  ret

PSGMOD_GET_PATTERN_NUMBER:
  ld a, (PSGMOD_SONG_POSITION)
  ld hl, (PSGMOD_SONG_START)
  ld c, a
  ld b, 0
  add hl, bc
  ld d, (HL)
  ret

PSGMOD_TOGGLE_SOUND:
  ld a, (PSGMOD_SOUND_DISABLED)
  cp 1
  jr Z, PSGMOD_ENABLE_SOUND

PSGMOD_DISABLE_SOUND:
  ld a, 159
  out ($7F), a
  ld a, 191
  out ($7F), a
  ld a, 223
  out ($7F), a
  ld a, 255
  out ($7F), a
  ld a, 1
  ld (PSGMOD_SOUND_DISABLED), a
  ret

PSGMOD_ENABLE_SOUND:
  xor a
  ld (PSGMOD_SOUND_DISABLED), a
  ld a, 1
  ld (PSGMOD_SPEED_COUNTER), a
  ret

HL_512:
  ld h, l
  ld l, 0
  add hl, hl
  ret
